#include "hart_modem.h"
#include "hart_serial.h"
#include "hart_lib.h"

//=========================== define ==========================================

//=========================== prototypes ======================================

// serial RX
void hart_modem_rxSerialRequest(uint16_t cmdId, uint8_t* payload, uint8_t len, hart_lib_master_t master);

//=========================== typedef =========================================

typedef struct
{
    // sending replies to Modem port
    uint8_t                    toModemBuf[HART_MAX_FRAME_LENGTH];

    // receiving cmds from Modem port
    uint8_t                    fromModemBuf[HART_MAX_FRAME_LENGTH];

    // forwarding cmds to mote
    uint8_t                   *toMoteBuf;
    int16_t                    toMoteBufLen;
		hart_modem_hartPayload_cbt toMoteCb;
    bool                       toMoteRequest;
    uint16_t                   toMoteCmdId;
} hart_modem_vars_t;

//=========================== variables =======================================

hart_modem_vars_t hart_modem_vars;

//=========================== public ==========================================

//========== admin

/**
\brief Setting up the instance.
*/
void    hart_modem_init(hart_modem_hartPayload_cbt cbt, uint8_t *hartPayloadBuf, uint8_t hartPayloadBufLen)
{
    // reset local variables
    memset(&hart_modem_vars,0,sizeof(hart_modem_vars));

    // store params
    hart_modem_vars.toMoteBuf    = hartPayloadBuf;
    hart_modem_vars.toMoteBufLen = hartPayloadBufLen;
		hart_modem_vars.toMoteCb     = cbt;

    // initialize the serial connection
    hart_serial_init(
        hart_modem_rxSerialRequest,
        hart_modem_vars.fromModemBuf,
        sizeof(hart_modem_vars.fromModemBuf)
    );
}

//========== serialRX

void hart_modem_rxSerialRequest(uint16_t cmdId, uint8_t* payload, uint8_t len, hart_lib_master_t master)
{
		int16_t responseLen;
	
    // check if the cmd is terminated by OEM MCU or by mote
    if (
        ((cmdId > 767)   && (cmdId < 1024)) || //SmartMesh WirelessHART Users' Guide p115
        ((cmdId > 64511) && (cmdId < 64766))
    )
    {
        // cmd terminated by mote
        DEBUG_MSG("hart MOTE cmd %d received, length: %d\n", cmdId, len);

        // save paramtere, flag request
        hart_modem_vars.toMoteCmdId      = cmdId;
        hart_modem_vars.toMoteRequest    = true;

        // build reply
				responseLen = hart_lib_cmdTerminatedByMote(
						cmdId, 
						payload, 
						len, 
						hart_modem_vars.toMoteBuf, 
						hart_modem_vars.toMoteBufLen
				);
			
				// send reply only if hart_lib returned no error
        if(responseLen > HART_LIB_DO_NOT_ANSWER)
        {
						hart_modem_vars.toMoteCb();			
        }
        else
        {           
						if (responseLen == HART_LIB_BUF_OVF)
						{										
								// communication with LTC5800 failed
								hart_serial_errorReply(31, HART_ERR_BUFF_OVF | HART_ERR_COMMUNICATION | HART_ERR_COM);
						}
        }
    }
    else
    {
        // all other cmds are terminated by this MCU
        DEBUG_MSG("hart OEM cmd %d received, length: %d\n", cmdId, len);

        // build reply
        responseLen = hart_lib_cmdTerminatedByMCU(
						cmdId, 
						payload, 
						len, 
						hart_modem_vars.toModemBuf, 
						sizeof(hart_modem_vars.toModemBuf),
						master
				);

        // send reply only if hart_lib returned no error
        if(responseLen > HART_LIB_DO_NOT_ANSWER)
        {
            hart_serial_txHartFrame(
                cmdId,                         // cmd id
                hart_modem_vars.toModemBuf,    // RC|data
                responseLen                    // RC+data
            );
        }
        else
				{
            DEBUG_MSG("Cannot answer\n");
				}
    }
}
void    hart_modem_hartPayloadReply(uint8_t bufLen, uint8_t *buf)
{
    uint16_t             receivedCmdId;
		hart_lib_wirelessPacket_t *pBuf;
    
		// copy to local buffer and init pointer
		memcpy(hart_modem_vars.toModemBuf, buf, bufLen);
		pBuf = (hart_lib_wirelessPacket_t*) hart_modem_vars.toModemBuf;
	
    // get received command id
    dn_read_uint16_t(&receivedCmdId, pBuf->cmdId);

    // am i expecting this answer?
    if(
        (hart_modem_vars.toMoteRequest == true) &&
        (hart_modem_vars.toMoteCmdId   == receivedCmdId) &&
        (bufLen                        >  3)
    )
    {
        // i AM expecting this answer

        // send reply
        hart_serial_txHartFrame(
            receivedCmdId,      // cmd
            pBuf->data,         // RC + data        
            pBuf->dataLen       // length of data
        );

        // clear request flag
        hart_modem_vars.toMoteRequest    = false;
    }
    else
    {
        // i am NOT expecting this answer 
        DEBUG_MSG("invalid hartPayload reply, rcvdCmdId=%d\n", receivedCmdId);
			
				// communication with LTC5800 failed
				hart_serial_errorReply(31, HART_ERR_COMMUNICATION | HART_ERR_COM); // cmd expansion flag, error code
        // increment error counter
        HART_PAYLOAD_ERR();
    }
}

