#include "app.h"
#include "time.h"

#include "../ADI/adxl363/adi_adxl363.h"
#include <arm_math.h>

//=========================== defines =========================================

//==== Accelerometer data rate
#define ACC_ODR                50 // in Hz

//==== make sure all hart constants are defined
#ifndef HART_PRIMARY_UNIT
#define HART_PRIMARY_UNIT      HART2_UNIT_NONE
#endif

#ifndef HART_SECONDARY_UNIT
#define HART_SECONDARY_UNIT    HART2_UNIT_NONE
#endif

#ifndef HART_TERTIARY_UNIT
#define HART_TERTIARY_UNIT     HART2_UNIT_NONE
#endif

#ifndef HART_QUATERNARY_UNIT
#define HART_QUATERNARY_UNIT   HART2_UNIT_NONE
#endif

#ifndef HART_PRIMARY_CLASS
#define HART_PRIMARY_CLASS     HART21_CLASS_NONE
#endif

#ifndef HART_SECONDARY_CLASS
#define HART_SECONDARY_CLASS   HART21_CLASS_NONE
#endif

#ifndef HART_TERTIARY_CLASS
#define HART_TERTIARY_CLASS    HART21_CLASS_NONE
#endif

#ifndef HART_QUATERNARY_CLASS
#define HART_QUATERNARY_CLASS  HART21_CLASS_NONE
#endif

//=========================== typedef =========================================

// data are read from ADXL363 FIFO in this order
typedef struct
{
    int16_t x;
    int16_t y;
    int16_t z;
    int16_t temperature;
} acc_fifoData_t;

//=========================== variables =======================================

//==== Device variables
app_deviceVar_t app_deviceVar[HART_MAX_DEV_VARS] = 
{
    {.unit              = HART_PRIMARY_UNIT,
     .classification    = HART_PRIMARY_CLASS}

#if (HART_MAX_DEV_VARS > 1)
    ,{.unit             = HART_SECONDARY_UNIT,
     .classification    = HART_SECONDARY_CLASS}
#endif

#if (HART_MAX_DEV_VARS > 2)
    ,{.unit             = HART_TERTIARY_UNIT,
     .classification    = HART_TERTIARY_CLASS}
#endif

#if (HART_MAX_DEV_VARS > 3)
    ,{.unit             = HART_QUATERNARY_UNIT,
     .classification    = HART_QUATERNARY_CLASS}
#endif
};

//==== Accelerometer variables
uint8_t             acc_mem[ADI_ADXL363_MEMORY_SIZE];        // Memory required for accelerometer driver
ADI_ADXL363_HANDLE  acc_hADXL363;                            // Accelerometer device handle
int16_t             acc_data[ADI_ADXL363_MAX_FIFO_SAMPLES];  /* Buffer to read samples */
uint16_t            acc_dataCount;                           /* Number of samples to read */

//=========================== prototypes ======================================

// Accelerometer routines
static  ADI_ADXL363_RESULT SetupAccel(void);                 // Opens and sets up the accelerometer driver
static  ADI_ADXL363_RESULT ReadFifoData(void);               // Reads the Fifo

// Helpers
float32_t rms(int16_t *value, uint8_t offset, uint16_t n);

//============================== public =======================================

app_deviceVar_t* app_getDeviceVar(uint8_t varNum)
{
    // check if passed parameter is valid
    if (varNum > (HART_MAX_DEV_VARS - 1))
        return NULL;
		
		// TODO DEBUG only
		app_deviceVar[0].value = 1.254;
		app_deviceVar[1].value = 100.4;
		app_deviceVar[2].value = -24.3;
		app_deviceVar[3].value = 0.0000456;		

    // return pointer to this var
    return &app_deviceVar[varNum];
}

float32_t   app_getLoopCurrent(void)
{
    // current signaling is not supported
    // TODO should this return something regardless???
    return 0;
}

int16_t app_enableMeasurement(uint8_t varNum, bool enable)
{
    int16_t returnCode = APP_OK;

    // find out which device variable we want to enable
    switch (varNum)
    {
    // TODO now we get all variables from ADXL363
    case HART_PRIMARY_VAR    :
        if (enable)
        {
            // Setup accelerometer
            if(SetupAccel() != ADI_ADXL363_SUCCESS)
            {
                // DEBUG_MSG("Accelerometer init failed!\n");
                returnCode = APP_ERROR;
            }
            else
            {
                DEBUG_MSG("Accelerometer enable success!\n");
            }
        }
        else
        {
            // disable adxl363
            adi_adxl363_Close(acc_hADXL363);
        }
        break;
    case HART_SECONDARY_VAR  :
    case HART_TERTIARY_VAR   :
    case HART_QUATERNARY_VAR :
    default:                    // passed parameter is invalid
        returnCode = APP_ERROR;
        break;
    }

    return returnCode;
}

//============================== private ======================================

//=============================================================================
//======================== ACC ROUTINES =======================================
//=============================================================================

/* Accelerometer callback - to indicate when data is ready */
static void AccelCallback(void *pCBParam, uint32_t Port, void *pin)
{
    float32_t           acc_rms[4];

    // ADXL363 datasheet p.9 TODO: offset should be processed as well!!!!
    // temperature measurement is very inaccurate
    float32_t           acc_constants[4] = {0.001030, 0.001030, 0.001030, 0.065};
    // number of complete data sets
    uint16_t        dataCount = acc_dataCount / (sizeof(acc_fifoData_t) / sizeof(int16_t));
    // pointer to acceleromtere dataset
    acc_fifoData_t  *acc_data = (acc_fifoData_t*) acc_data;

    // Read the fifo
    // TODO, this is now blocking!
    ReadFifoData();

    // TODO data processing should be moved to DMA finished transfer interrupt
    // calculate RMS values of all data
    acc_rms[0] = rms(&acc_data->x, 4, dataCount);
    acc_rms[1] = rms(&acc_data->y, 4, dataCount);
    acc_rms[2] = rms(&acc_data->z, 4, dataCount);
    acc_rms[3] = rms(&acc_data->temperature, 4, dataCount);

    // convert to g and Celsius degree
    arm_mult_f32(acc_rms, acc_constants, acc_rms, 4);

    // flag new data, save the values
    for (uint8_t i = HART_PRIMARY_VAR; i < HART_MAX_DEV_VARS; i++)
    {
        app_deviceVar[i].value       = acc_rms[i];
        app_deviceVar[i].isOffLimits = false;
        app_deviceVar[i].newData     = true;
    }

    DEBUG_MSG("Acc INT %d\n", dataCount);

    return;
}

/* Opens and sets up the accelerometer driver */
static ADI_ADXL363_RESULT SetupAccel(void)
{
    /* Open the accelerometer driver */
    if(adi_adxl363_Open (acc_mem,
                         ADI_ADXL363_MEMORY_SIZE,
                         &acc_hADXL363 ) != ADI_ADXL363_SUCCESS)
    {
        //DEBUG_MSG("Failed to open the accelerometer driver \n");
        return ADI_ADXL363_FAILURE;
    }

    // enter stand-by mode
    if(adi_adxl363_Enable(acc_hADXL363,
                          false) != ADI_ADXL363_SUCCESS)
    {
        //DEBUG_MSG("Failed to ENTER STAND-BY MODE \n");
        return ADI_ADXL363_FAILURE;
    }

    /* Register a callback for the accelerometer */
    if( adi_adxl363_RegisterCallback(acc_hADXL363,
                                     AccelCallback,
                                     NULL) != ADI_ADXL363_SUCCESS)
    {
        //DEBUG_MSG("Failed to register callback \n");
        return ADI_ADXL363_FAILURE;
    }

    /* Set the GPIO pins connected to the host for accelerometer interrupts */
    if(adi_adxl363_ConfigIntPin (acc_hADXL363,
                                 ADI_XINT_EVENT_INT3,
                                 ADI_ADXL363_INTPIN1) != ADI_ADXL363_SUCCESS)
    {
        //DEBUG_MSG("Failed to configure interrupt pin1 \n");
        return ADI_ADXL363_FAILURE;
    }

    // Start measurements
    if(adi_adxl363_Enable(acc_hADXL363, true) != ADI_ADXL363_SUCCESS)
    {
        DEBUG_MSG("Failed to enable accelerometer \n");
        return ADI_ADXL363_FAILURE;
    }
		
	  // Start Sensor strobe service
		time_configureSensorStrobe(ACC_ODR);

    return ADI_ADXL363_SUCCESS;
}

/* Configure the Fifo, and read blocks of data when a watermark is
 * reached. */
static ADI_ADXL363_RESULT ReadFifoData(void)
{
    uint8_t  ints;               /* Interrupts fired */

    /* Get the source(s) of interrupt - also clears the interrupts */
    if(adi_adxl363_GetIntStatus (acc_hADXL363,
                                 &ints) != ADI_ADXL363_SUCCESS )
    {
        // DEBUG_MSG("Failed to get interrupt source \n");
    }

    /* Have we reached the FIFO watermark? */
    if((ints & ADI_ADXL363_INT_WATERMARK) == ADI_ADXL363_INT_WATERMARK)
    {
        /* Get the count - may contain more than the watermark number */
        if(ADI_ADXL363_SUCCESS == adi_adxl363_GetEntriesInFIFO(acc_hADXL363, &acc_dataCount))
        {
            /* Make sure there are samples to read */
            if(acc_dataCount != 0)
            {
                /* Read the FIFO */
                if(ADI_ADXL363_SUCCESS != adi_adxl363_ReadFIFO(acc_hADXL363,
                        acc_data,
                        acc_dataCount))
                {
                    // DEBUG_MSG("Could not collect FIFO data \n");
                    return ADI_ADXL363_FAILURE;
                }
            }
            else
            {
                //DEBUG_MSG("Fifo does not contain any data \n");
                return ADI_ADXL363_FAILURE;
            }
        }
        //  DEBUG_MSG("Fifo successfully emptied\n");
    }
    return ADI_ADXL363_SUCCESS;
}

//============================== helpers ======================================

float32_t rms(int16_t *value, uint8_t offset, uint16_t n)
{
    float32_t rms;
    int16_t   temp;
    uint32_t  sum_squares = 0;

    for (uint16_t i = 0; i < n; i++)
    {
        // b12 and b13 are sign extension bits
        // b15 and b14 are data type bits ADXL363 datasheet p.40
        if ((*value) & 0x3000)
            temp = (*value) | 0xC000;
        else
            temp = (*value) & (~0xC000);

        sum_squares += temp * temp; // square of values
        value += offset;            // move to next value
    }

    // divide by number of samples
    sum_squares /= n;

    // square root
    arm_sqrt_f32((float32_t) sum_squares, &rms);

    return rms;
}
